import { execSync } from 'child_process';
import { readFileSync, existsSync } from 'fs';
import firebase from 'firebase/compat/app';
import 'firebase/compat/firestore';
import 'firebase/compat/auth';
import { intro, outro, spinner } from '@clack/prompts';
import capitalizedName from './utils/getUserInfo.js';
import colorText from './utils/textUtils.js';

const s = spinner();

const firebaseConfig = {
    apiKey: "AIzaSyABm9M7XceFH6pSvvbMuRJw3n5nBeak3L0",
    authDomain: "share-brewfiles.firebaseapp.com",
    projectId: "share-brewfiles",
    storageBucket: "share-brewfiles.appspot.com",
    messagingSenderId: "1071283599196",
    appId: "1:1071283599196:web:a4910843f1bae76d7af399",
    measurementId: "G-WGD9RH2E7M"
  };

firebase.initializeApp(firebaseConfig);

function isHomebrewInstalled() {
    try {
        execSync('brew --version');
        return true;
    } catch (error) {
        return false;
    }
}

async function startClackSpinner(text) {
    s.start(text)
    await new Promise(resolve => setTimeout(resolve, 2000));
}

async function stopClackSpinner(text) {
    s.stop(text)
    await new Promise(resolve => setTimeout(resolve, 1000));
}

function installHomebrew() {
    execSync('/bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"');
}

async function createNewBrewfile() {
    execSync('brew bundle dump');
}

async function getBrewfile() {
    await startClackSpinner('Checking for existing Brewfile')
    if (existsSync('Brewfile')) {
        await stopClackSpinner("Found existing Brewfile!")
    } else {
        await stopClackSpinner("No existing Brewfile found!")
        await startClackSpinner("Generating Brewfile for you")
        createNewBrewfile()
        await stopClackSpinner("Finished generating new Brewfile!")
    }
}

async function readBrewfileData() {
    const brewfileData = readFileSync('Brewfile', 'utf-8');
    return brewfileData;
}

async function uploadBrewfileToFirebase(brewfileData) {
    const db = firebase.firestore();
    const collectionRef = db.collection('brewfiles');
    await collectionRef.add({ data: brewfileData });
}

async function parseBrewfile(brewfileData) {
    const packages = {};
    const lines = brewfileData.split('\n');
    
    lines.forEach((line, index) => {
        if (line.startsWith('tap')) {
            const [, name] = line.split(' ');
            packages[index + 1] = { name, packageManager: 'tap' };
        } else if (line.startsWith('brew')) {
            const [, name] = line.split(' ');
            packages[index + 1] = { name, packageManager: 'brew' };
        } else if (line.startsWith('vscode')) {
            const [, name] = line.split(' ');
            packages[index + 1] = { name, packageManager: 'vscode' };
        }
    });
    return packages;
}

async function main() {
    console.clear();
    console.log('📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦')
    console.log()
    intro(`Welcome to ${colorText('Share Brewfiles', "BLACK", "GREEN")}, ${capitalizedName}!`)
    if (!isHomebrewInstalled()) {
        installHomebrew();
    } else {
        await getBrewfile();
        await startClackSpinner("Uploading Brewfile to our website")
        const brewfileData = await readBrewfileData();
        if (brewfileData) {
            const packages = await parseBrewfile(brewfileData);
            await uploadBrewfileToFirebase(packages);
        }
    }
    await stopClackSpinner("Successfully uploaded your Brewfile!")
    outro(`${colorText("Done! Now check out the community leaderboard here:", "BLACK", "GREEN")}`)
    console.log(`👉 ${colorText("https://share-brewfiles-website.vercel.app/", "WHITE", "MAGENTA")} 👈`)
    console.log()
    console.log('📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦 📦')
    process.exit(0);
}

main().catch(console.error);
